#########################################################################################################
#				StudiAISV_draw-waveform-sgram-f0-tg.praat				#
# 		Copyright (C) 2020 Vincenzo Galatà, galata.vincenzo [at] gmail.com - 04/05/2020 	#
# 			Distribution under the GNU General Public License.				#
#													#
# This program is free software; you can redistribute it and/or modify it under the terms of the 	#
# GNU General Public License as published by the Free Software Foundation; either version 2 of the 	#
# License, or (at your option) any later version.							#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 	#
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 			#
# See the GNU General Public License for more details: https://opensource.org/licenses/GPL-2.0		#
# You should have received a copy of the GNU General Public License along with this program; if not,	#
# write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. #
#			*********************************************					#
### 
### This script draws and saves a Praat figures as a 600DPI *.png file with the size in cm given in the form.
### Some options can be set in the form.
### The script requires the TextGrid and the sound file to be open and selected in the Object window.
### Before running the script, I recommend you create the two required files from the TextGrid Editor 
### by selecting and exporting the desired portion for which you are plotting the image (e.g. 
### File > Extract selected sound (preserve times), File > Extract selected TextGrid (preserve times) and
### preferably save the two objects (for future needs). 
### The script automatically manages TextGrids with up to three Tiers (more than three makes the figure difficult to read).
### Script tested and working under Praat 6.0.37
### 
### Modified by V. Galatà from the original version by Pauline Welby (draw-waveform-sgram-f0.praat)
### welby@ling.ohio-state.edu, welby [at] icp.inpg.fr | April 15, 2003 | Last change, November 20, 2005 | Praat 4.3.29
###
################################################################

################################################################
#BEGINNING OF SCRIPT
################################################################

form draw-waveform-sgram-f0-tg
	sentence Enter_directory_where_figure_will_be_saved E:\Vincenzo\Desktop/
	sentence Figure_Title FigureName
	boolean Save_as_600DPI_png_file yes
	boolean Superimpose_f0_curve yes
	positive f0min 70
	positive f0max 200
	sentence Color_for_pitch_line Red
	boolean Use_advanced_F0_settings_with_ac_method 1
	integer smooth 10
	integer timestep 0
	integer maxcandidates 15
	real silenceThr 0.03
	real voicingThr 0.45
	real octave 0.01
	real octavejump 0.35
	real voiceunvoiced 0.14
	boolean interpolate no
	comment Settings for the image:
	sentence Enter_y_axis_label Fundamental frequency (Hz)
	positive Enter_x_axis_major_time_unit 0.5
	positive Enter_x_axis_minor_time_unit 0.1
	positive Enter_spectrogram_maximum_frequency 8000
	positive Enter_width_of_desired_figure_in_cm 12
	positive Enter_height_of_desired_figure_in_cm 8
endform

# shorten variable names
figDir$ = enter_directory_where_figure_will_be_saved$
superimp_f0 = superimpose_f0_curve
color$ = color_for_pitch_line$
advanced = use_advanced_F0_settings_with_ac_method
yaxis$ = enter_y_axis_label$
timeMajUnit = enter_x_axis_major_time_unit
timeMinUnit = enter_x_axis_minor_time_unit
hzmax = enter_spectrogram_maximum_frequency
rightBound = enter_width_of_desired_figure_in_cm
tgridBot = enter_height_of_desired_figure_in_cm
png = save_as_600DPI_png_file

# Converts cm to inches for Praat's picture window
rightBound = rightBound / 2.54
tgridBot = tgridBot / 2.54

pause Please make sure Sound and TextGrid are open and selected in the Object window.

# The two objects need to be already selected in the object window.
soundID$ = selected$("Sound")
textGridID$ = selected$("TextGrid")

select Sound 'soundID$'
# Make Pitch object
if superimp_f0 = 1
	if advanced = 0
		To Pitch... 0.005 75 600
	else
		To Pitch (ac): timestep, f0min, maxcandidates, "no", silenceThr, 
           	...voicingThr, octave, octavejump, voiceunvoiced, f0max
		Smooth: 10
		if interpolate = 1
			Interpolate
		endif
	endif
endif

# Make Spectrogram object
select Sound 'soundID$'
To Spectrogram... 0.005 'hzmax' 0.002 20 Gaussian

# Get number of tiers to manage the Picture viewport settings.
select TextGrid 'textGridID$'
ntiers = Get number of tiers

# Specify font type size, color
Times
Font size... 18/2.54
Black
Erase all

# Define size and position of waveform (by specifying grid coordinates)
Viewport... 0 'rightBound' 0 'tgridBot'/100*35

# Draw waveform
select Sound 'soundID$'
Draw... 0 0 0 0 no curve
 
# Define size and position of spectrogram depending on number of tiers.
if ntiers = 3
	Viewport... 0 'rightBound' 'tgridBot'/100*20 'tgridBot'/100*63
elsif ntiers = 2
	Viewport... 0 'rightBound' 'tgridBot'/100*20 'tgridBot'/100*70
elsif ntiers = 1
	Viewport... 0 'rightBound' 'tgridBot'/100*20 'tgridBot'/100*80
endif
      
# Draw spectrogram
select Spectrogram 'soundID$'
Paint... 0 0 0 'hzmax' 100 yes 50 6 0 no
Line width... 1
One mark left... 'hzmax' no no yes
One mark left... 1 no no yes
Marks left every... 1 2000 yes yes no
Text left... yes Frequency (Hz)

if superimp_f0 = 1
	# Draw Pitch curve first as a thick white line
	select Pitch 'soundID$'
	Line width... 6
	White
	Draw... 0 0 'f0min' 'f0max' no

	# then as a thin black line
	Line width... 1.5
	'color$'
	Draw... 0 0 'f0min' 'f0max' no

	# Label y axis
	# N.B.: can change language of labels here. 
	# Also, Praat default label for y axis is "Pitch".
	Line width... 1
	One mark right... 'f0max' no no yes
	One mark right... 'f0min' no no yes
	Marks right every... 1 50 yes yes no
	Text right... yes 'yaxis$'
	Draw... 0 0 'f0min' 'f0max' no
endif

Black

# Label x axis 
# Text bottom... yes 'xaxis$'
#Marks bottom every... 1 'timeMinUnit' no yes no
#Marks bottom every... 1 'timeMajUnit' yes yes no

select TextGrid 'textGridID$'

# Define size and position of TextGrid
Viewport... 0 'rightBound' 0 'tgridBot'
Marks bottom every... 1 'timeMinUnit' no yes no
Marks bottom every... 1 'timeMajUnit' yes yes no

# Draw TextGrid
Draw... 0 0 yes no yes

# Define size and position of inner box
Viewport... 0 'rightBound' 0 'tgridBot'

Black
#Draw inner box
Text top... yes 'figure_Title$'

# Write to a file (see choices under File in the Praat picture window for more options)
if 'png' = 1
	Save as 600-dpi PNG file... 'figDir$''textGridID$'_'figure_Title$'.png
endif

# Remove unneeded objects from Praat objects list
select Spectrogram 'soundID$'
Remove

# Remove Pitch object, if necessary
if superimp_f0 = 1
	select all
	minus Sound 'soundID$'
	minus TextGrid 'textGridID$'
	Remove
endif

select Sound 'soundID$'
plus TextGrid 'textGridID$'

################################################################
#END OF SCRIPT
################################################################